"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  H: () => H,
  HIGHLIGHT_REQUEST_HEADER: () => HIGHLIGHT_REQUEST_HEADER,
  Handlers: () => handlers_exports,
  Highlight: () => Highlight
});
module.exports = __toCommonJS(src_exports);

// src/log.ts
var start = Date.now();
function log(from, ...data) {
  const prefix = `[Highlight Node.js ${(Date.now() - start) / 1e3}s] - {${from}}`;
  console.log.apply(console, [prefix, ...data]);
}

// src/sdk.ts
var HIGHLIGHT_REQUEST_HEADER = "x-highlight-request";
var _debug = false;
var highlight_obj;
var H = {
  init: (options) => {
    _debug = !!options.debug;
    try {
      highlight_obj = new Highlight(options);
    } catch (e) {
      console.warn("highlight-node init error: ", e);
    }
  },
  stop: async () => {
    if (!highlight_obj) {
      return;
    }
    try {
      await highlight_obj.stop();
    } catch (e) {
      console.warn("highlight-node stop error: ", e);
    }
  },
  isInitialized: () => !!highlight_obj,
  consumeError: (error, secureSessionId, requestId, metadata) => {
    try {
      highlight_obj.consumeCustomError(
        error,
        secureSessionId,
        requestId,
        metadata
      );
    } catch (e) {
      console.warn("highlight-node consumeError error: ", e);
    }
  },
  recordMetric: (secureSessionId, name, value, requestId, tags) => {
    try {
      highlight_obj.recordMetric(
        secureSessionId,
        name,
        value,
        requestId,
        tags
      );
    } catch (e) {
      console.warn("highlight-node recordMetric error: ", e);
    }
  },
  flush: async () => {
    try {
      await highlight_obj.flush();
    } catch (e) {
      console.warn("highlight-node flush error: ", e);
    }
  },
  log: (message, level, secureSessionId, requestId, metadata) => {
    const o = { stack: {} };
    Error.captureStackTrace(o);
    try {
      highlight_obj.log(
        new Date(),
        message,
        level,
        o.stack,
        secureSessionId,
        requestId,
        metadata
      );
    } catch (e) {
      console.warn("highlight-node log error: ", e);
    }
  },
  parseHeaders: (headers) => {
    try {
      if (headers && headers[HIGHLIGHT_REQUEST_HEADER]) {
        const [secureSessionId, requestId] = `${headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
        return { secureSessionId, requestId };
      } else {
        H._debug(
          `request headers do not contain ${HIGHLIGHT_REQUEST_HEADER}`
        );
      }
    } catch (e) {
      console.warn("highlight-node parseHeaders error: ", e);
    }
    return void 0;
  },
  _debug: (...data) => {
    if (_debug) {
      log("H", ...data);
    }
  }
};

// src/client.ts
var import_sdk_trace_base = require("@opentelemetry/sdk-trace-base");
var import_api = require("@opentelemetry/api");
var import_sdk_node = require("@opentelemetry/sdk-node");
var import_exporter_trace_otlp_http = require("@opentelemetry/exporter-trace-otlp-http");
var import_auto_instrumentations_node = require("@opentelemetry/auto-instrumentations-node");

// src/hooks.ts
var ConsoleLevels = {
  debug: "debug",
  info: "info",
  log: "info",
  count: "info",
  dir: "info",
  warn: "warn",
  assert: "warn",
  error: "error",
  trace: "trace"
};
var consoleHooked = false;
function hookConsole(methodsToRecord, cb) {
  if (consoleHooked)
    return;
  consoleHooked = true;
  for (const [level, highlightLevel] of Object.entries(ConsoleLevels)) {
    if ((methodsToRecord == null ? void 0 : methodsToRecord.length) && methodsToRecord.indexOf(level) === -1) {
      continue;
    }
    const origWrite = console[level];
    console[level] = function(...data) {
      const date = new Date();
      try {
        return origWrite(...data);
      } finally {
        const o = { stack: {} };
        Error.captureStackTrace(o);
        cb({
          date,
          level: highlightLevel,
          message: data.map(
            (o2) => typeof o2 === "object" ? JSON.stringify(o2) : o2
          ).join(" "),
          stack: o.stack
        });
      }
    };
  }
}

// src/client.ts
var import_timers = require("timers");
var import_semantic_conventions = require("@opentelemetry/semantic-conventions");
var import_resources = require("@opentelemetry/resources");
var OTLP_HTTP = "https://otel.highlight.io:4318";
var Highlight = class {
  FLUSH_TIMEOUT = 10;
  _intervalFunction;
  _projectID;
  _debug;
  otel;
  tracer;
  processor;
  constructor(options) {
    this._debug = !!options.debug;
    this._projectID = options.projectID;
    if (!options.disableConsoleRecording) {
      hookConsole(options.consoleMethodsToRecord, (c) => {
        this.log(c.date, c.message, c.level, c.stack);
      });
    }
    if (!this._projectID) {
      console.warn(
        "Highlight project id was not provided. Data will not be recorded."
      );
    }
    this.tracer = import_api.trace.getTracer("highlight-node");
    const exporter = new import_exporter_trace_otlp_http.OTLPTraceExporter({
      url: `${options.otlpEndpoint ?? OTLP_HTTP}/v1/traces`
    });
    this.processor = new import_sdk_trace_base.BatchSpanProcessor(exporter, {});
    const attributes = {};
    attributes["highlight.project_id"] = this._projectID;
    if (options.serviceName) {
      attributes[import_semantic_conventions.SemanticResourceAttributes.SERVICE_NAME] = options.serviceName;
    }
    if (options.serviceVersion) {
      attributes[import_semantic_conventions.SemanticResourceAttributes.SERVICE_VERSION] = options.serviceVersion;
    }
    this.otel = new import_sdk_node.NodeSDK({
      autoDetectResources: true,
      resourceDetectors: [import_resources.processDetectorSync],
      resource: {
        attributes,
        merge: (resource) => new import_resources.Resource({
          ...(resource == null ? void 0 : resource.attributes) ?? {},
          ...attributes
        })
      },
      spanProcessor: this.processor,
      traceExporter: exporter,
      instrumentations: [
        (0, import_auto_instrumentations_node.getNodeAutoInstrumentations)({
          "@opentelemetry/instrumentation-fs": {
            enabled: options.enableFsInstrumentation ?? false
          }
        })
      ]
    });
    this.otel.start();
    this._intervalFunction = setInterval(
      () => this.flush(),
      this.FLUSH_TIMEOUT * 1e3
    );
    for (const event of [
      "beforeExit",
      "exit",
      "SIGABRT",
      "SIGTERM",
      "SIGINT"
    ]) {
      process.on(event, async () => {
        await this.flush();
      });
    }
    this._log(`Initialized SDK for project ${this._projectID}`);
  }
  async stop() {
    await this.flush();
    await this.otel.shutdown();
    if (this._intervalFunction) {
      (0, import_timers.clearInterval)(this._intervalFunction);
    }
  }
  _log(...data) {
    if (this._debug) {
      log("client", ...data);
    }
  }
  recordMetric(secureSessionId, name, value, requestId, tags) {
    if (!this.tracer)
      return;
    const span = this.tracer.startSpan("highlight-ctx");
    span.addEvent("metric", {
      ["highlight.project_id"]: this._projectID,
      ["metric.name"]: name,
      ["metric.value"]: value,
      ...secureSessionId ? {
        ["highlight.session_id"]: secureSessionId
      } : {},
      ...requestId ? {
        ["highlight.trace_id"]: requestId
      } : {}
    });
    for (const t of tags || []) {
      span.setAttribute(t.name, t.value);
    }
    span.end();
  }
  log(date, msg, level, stack, secureSessionId, requestId, metadata) {
    if (!this.tracer)
      return;
    const span = this.tracer.startSpan("highlight-ctx");
    span.addEvent(
      "log",
      {
        ...metadata ?? {},
        ["exception.stacktrace"]: JSON.stringify(stack),
        ["highlight.project_id"]: this._projectID,
        ["log.severity"]: level,
        ["log.message"]: msg,
        ...secureSessionId ? {
          ["highlight.session_id"]: secureSessionId
        } : {},
        ...requestId ? {
          ["highlight.trace_id"]: requestId
        } : {}
      },
      date
    );
    span.end();
  }
  consumeCustomError(error, secureSessionId, requestId, metadata) {
    const span = this.tracer.startSpan("highlight-ctx");
    span.recordException(error);
    if (metadata != void 0) {
      span.setAttributes(metadata);
    }
    if (secureSessionId) {
      span.setAttributes({ ["highlight.session_id"]: secureSessionId });
    }
    if (requestId) {
      span.setAttributes({ ["highlight.trace_id"]: requestId });
    }
    this._log("created error span", span);
    span.end();
  }
  async flush() {
    await this.processor.forceFlush().catch((e) => console.warn("highlight-node failed to flush: ", e));
  }
};

// src/handlers.ts
var handlers_exports = {};
__export(handlers_exports, {
  errorHandler: () => errorHandler,
  firebaseCallableFunctionHandler: () => firebaseCallableFunctionHandler,
  firebaseHttpFunctionHandler: () => firebaseHttpFunctionHandler,
  serverlessFunction: () => serverlessFunction,
  trpcOnError: () => trpcOnError
});
function processErrorImpl(options, req, error, metadata) {
  let secureSessionId;
  let requestId;
  if (req.headers && req.headers[HIGHLIGHT_REQUEST_HEADER]) {
    ;
    [secureSessionId, requestId] = `${req.headers[HIGHLIGHT_REQUEST_HEADER]}`.split("/");
  }
  H._debug("processError", "extracted from headers", {
    secureSessionId,
    requestId
  });
  if (!H.isInitialized()) {
    H.init(options);
    H._debug("initialized H");
  }
  H.consumeError(error, secureSessionId, requestId, metadata);
  H._debug("consumed error", error);
}
function errorHandler(options, metadata) {
  H._debug("setting up error handler");
  return (error, req, res, next) => {
    H._debug("handling request");
    try {
      processErrorImpl(options, req, error, metadata);
    } finally {
      next(error);
    }
  };
}
async function trpcOnError({
  error,
  req
}, options, metadata) {
  try {
    if (!H.isInitialized()) {
      H.init(options);
    }
    processErrorImpl(options, req, error, metadata);
    await H.flush();
  } catch (e) {
    console.warn("highlight-node trpcOnError error:", e);
  }
}
var makeHandler = (origHandler, options, metadata, headersExtractor) => {
  return async (...args) => {
    try {
      return await origHandler(...args);
    } catch (e) {
      try {
        if (e instanceof Error) {
          if (!H.isInitialized()) {
            H.init(options);
          }
          processErrorImpl(
            options,
            { headers: headersExtractor(args) },
            e,
            metadata
          );
          await H.flush();
        }
      } catch (e2) {
        console.warn("highlight-node serverlessFunction error:", e2);
      }
      throw e;
    }
  };
};
function firebaseHttpFunctionHandler(origHandler, options, metadata) {
  return makeHandler(
    origHandler,
    options,
    metadata,
    (req, res) => req.headers
  );
}
function firebaseCallableFunctionHandler(origHandler, options, metadata) {
  return makeHandler(
    origHandler,
    options,
    metadata,
    (data, ctx) => ctx.rawRequest
  );
}
function serverlessFunction(origHandler, options, metadata) {
  return makeHandler(
    origHandler,
    options,
    metadata,
    (event) => event == null ? void 0 : event.headers
  );
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  H,
  HIGHLIGHT_REQUEST_HEADER,
  Handlers,
  Highlight
});
